@@ -6,7 +6,7 @@ import numpy as np
 import seaborn as sns
 import statsmodels as sm
 from sklearn.inspection import permutation_importance
-from .utils import ifelse
+from .utils import ifelse, intersect, setdiff
 from .regression import sim_prediction
 from .perf import auc
 
@@ -28,8 +28,19 @@ def distr_plot(df: pd.DataFrame, cols: list = None, nint: int = 25, **kwargs):
         series will be treated as a categorical variable
     **kwargs : Named arguments to be passed to the pandas plotting methods
     """
-    if cols != None:
-        df = df[cols]
+    if cols is None:
+        all_cols = df.columns
+        cols = []
+        for i, c in enumerate(all_cols):
+            if (
+                not pd.api.types.is_numeric_dtype(df[c].dtype)
+                and not pd.api.types.is_categorical_dtype(df[c].dtype)
+            ) or pd.api.types.is_object_dtype(df[c].dtype):
+                print(f"No plot will be created for {c} (type {df[c].dtype})")
+            else:
+                cols.append(c)
+
+    df = df.loc[:, cols].copy()
 
     fig, axes = plt.subplots(
         max(math.ceil(df.shape[1] / 2), 2), 2, figsize=(10, 2 * max(df.shape[1], 4))
@@ -40,7 +51,7 @@ def distr_plot(df: pd.DataFrame, cols: list = None, nint: int = 25, **kwargs):
         s = df[c]
         j = ifelse(i % 2 == 0, 0, 1)
         if pd.api.types.is_integer_dtype(s.dtype) and s.nunique() < nint:
-            s.value_counts(sort=False).plot.bar(
+            s.sort_values().value_counts(sort=False).plot.bar(
                 ax=axes[row, j], title=c, rot=0, color="slateblue", **kwargs
             )
         elif pd.api.types.is_numeric_dtype(s.dtype):
@@ -50,7 +61,7 @@ def distr_plot(df: pd.DataFrame, cols: list = None, nint: int = 25, **kwargs):
                 ax=axes[row, j], title=c, rot=0, color="slateblue", **kwargs
             )
         else:
-            print(f"No plot for {c} (type {s.dtype})")
+            print(f"No plot will be created for {c} (type {s.dtype})")
 
         if j == 1:
             row += 1
@@ -145,12 +156,17 @@ def pred_plot_sm(
         # legacy for when only an un-fitted model was accepted
         model = fitted
 
-    min_max = [np.Inf, -np.Inf]
+    # margin to add above and below in plots
+    plot_margin = 0.025
 
     def calc_ylim(lab, lst, min_max):
         if isinstance(fix, bool) and fix == True:
             vals = lst[lab]
-            return (min(min_max[0], min(vals)), max(min_max[1], max(vals)))
+            min_vals = min(vals)
+            max_vals = max(vals)
+            mmin = min(min_max[0], min_vals - plot_margin * min_vals)
+            mmax = max(min_max[1], max_vals + plot_margin * max_vals)
+            return (mmin, mmax)
         elif not isinstance(fix, bool) and len(fix) == 2:
             return fix
         else:
@@ -162,6 +178,9 @@ def pred_plot_sm(
 
     if not isinstance(hline, float) and not isinstance(hline, int):
         hline = False
+        min_max = (np.Inf, -np.Inf)
+    else:
+        min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
 
     if incl is None:
         incl = extract_evars(model, df.columns)
@@ -189,7 +208,10 @@ def pred_plot_sm(
 
     for v in incl_int:
         vl = v.split(":")
-        is_num = [pd.api.types.is_numeric_dtype(df[c].dtype) for c in vl]
+        is_num = [
+            pd.api.types.is_numeric_dtype(df[c].dtype) and df[c].nunique() > 5
+            for c in vl
+        ]
         iplot = sim_prediction(df, vary=vl, nnv=nnv, minq=minq, maxq=maxq)
         iplot["prediction"] = fitted.predict(iplot)
         if sum(is_num) < 2:
@@ -199,25 +221,28 @@ def pred_plot_sm(
     row = col = 0
     for i, v in enumerate(incl):
         col = ifelse(i % 2 == 0, 0, 1)
-        if pd.api.types.is_numeric_dtype(df[v].dtype):
+        if pd.api.types.is_numeric_dtype(df[v].dtype) and df[v].nunique() > 5:
             fig = sns.lineplot(x=v, y="prediction", data=pred_dict[v], ax=ax[row, col])
         else:
             fig = sns.lineplot(
                 x=v, y="prediction", marker="o", data=pred_dict[v], ax=ax[row, col]
             )
         if isinstance(min_max, tuple) and len(min_max) == 2:
-            ax[row, col].set(ylim=tuple(min_max))
+            ax[row, col].set(ylim=min_max)
         if hline != False:
             ax[row, col].axhline(y=hline, linestyle="--")
 
         if col == 1:
             row += 1
 
-    start_col = ifelse(col == 1, 0, 1)
+    start_col = ifelse((col == 1) or len(incl) == 0, 0, 1)
     for j, v in enumerate(incl_int):
         col = ifelse(j % 2 == start_col, 0, 1)
         vl = v.split(":")
-        is_num = [pd.api.types.is_numeric_dtype(df[c].dtype) for c in vl]
+        is_num = [
+            pd.api.types.is_numeric_dtype(df[c].dtype) and df[c].nunique() > 5
+            for c in vl
+        ]
         if sum(is_num) == 2:
             plot_data = (
                 pred_dict[v]
@@ -246,7 +271,7 @@ def pred_plot_sm(
             )
 
         if isinstance(min_max, tuple) and len(min_max) == 2 and sum(is_num) < 2:
-            ax[row, col].set(ylim=tuple(min_max))
+            ax[row, col].set(ylim=min_max)
         if sum(is_num) < 2 and hline != False:
             ax[row, col].axhline(y=hline, linestyle="--")
 
@@ -268,6 +293,7 @@ def pred_plot_sm(
 def pred_plot_sk(
     fitted,
     df,
+    transformed=None,
     rvar=None,
     incl=None,
     excl=[],
@@ -285,7 +311,10 @@ def pred_plot_sk(
     Parameters
     ----------
     fitted : A fitted sklearn model
-    df : Pandas DataFrame with data used for estimation
+    df : Pandas DataFrame with data used for estimatin. Should include categorical variables
+        in their original for (i.e., before using get_dummies)
+    transformed : List of column names that were transformed using Pandas' get_dummies. If
+        None, the function will try to determine which variables might have been transformed
     rvar : The column name for the response/target variable
     incl : A list of column names to generate prediction plots for. If None, all
         variables will be plotted
@@ -311,7 +340,33 @@ def pred_plot_sk(
     fn = fitted.feature_names_in_
 
     not_transformed = [c for c in df.columns for f in fn if c == f]
-    transformed = list(set([c for c in df.columns for f in fn if c in f and c != f]))
+    if transformed is None:
+        transformed = list(
+            set([c for c in df.columns for f in fn if c in f and c != f])
+        )
+
+    not_transformed = [c for c in df.columns for f in fn if c == f]
+    ints = intersect(transformed, not_transformed)
+    if len(ints) > 0:
+        trn = '", "'.join(transformed)
+        ints = ", ".join(ints)
+        not_transformed = setdiff(not_transformed, transformed)
+        mess = f"""It is not clear which variables were (not) transformed using get_dummies. Please specify the
+        transformed variables in your call to pred_plot_sk as `transformed=["{trn}"]`.
+        In particular it is unclear how {ints} should be treated. Please remove entries from the list that
+        are not correct and add any that are missing"""
+        raise Exception(mess)
+
+    if hasattr(fitted, "classes_"):
+        sk_type = "classification"
+    else:
+        sk_type = "regression"
+
+    def pred_fun(fitted, df):
+        if sk_type == "classification":
+            return fitted.predict_proba(df)[:, 1]
+        else:
+            return fitted.predict(df)
 
     if incl is None:
         incl = not_transformed + transformed
@@ -324,17 +379,26 @@ def pred_plot_sk(
         else:
             return df
 
-    min_max = [np.Inf, -np.Inf]
     if rvar is not None and isinstance(hline, bool) and hline == True:
         hline = df[rvar].mean()
 
+    # margin to add above and below in plots
+    plot_margin = 0.025
+
     if not isinstance(hline, float) and not isinstance(hline, int):
         hline = False
+        min_max = (np.Inf, -np.Inf)
+    else:
+        min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
 
     def calc_ylim(lab, lst, min_max):
         if isinstance(fix, bool) and fix == True:
             vals = lst[lab]
-            return (min(min_max[0], min(vals)), max(min_max[1], max(vals)))
+            min_vals = min(vals)
+            max_vals = max(vals)
+            mmin = min(min_max[0], min_vals - plot_margin * min_vals)
+            mmax = max(min_max[1], max_vals + plot_margin * max_vals)
+            return (mmin, mmax)
         elif not isinstance(fix, bool) and len(fix) == 2:
             return fix
         else:
@@ -362,13 +426,16 @@ def pred_plot_sk(
             maxq=maxq,
         )
         iplot_dum = dummify(iplot, transformed)[fn]
-        iplot["prediction"] = fitted.predict_proba(iplot_dum)[:, 1]
+        iplot["prediction"] = pred_fun(fitted, iplot_dum)
         min_max = calc_ylim("prediction", iplot, min_max)
         pred_dict[v] = iplot
 
     for v in incl_int:
         vl = v.split(":")
-        is_num = [pd.api.types.is_numeric_dtype(df[c].dtype) for c in vl]
+        is_num = [
+            pd.api.types.is_numeric_dtype(df[c].dtype) and df[c].nunique() > 5
+            for c in vl
+        ]
         iplot = sim_prediction(
             df[transformed + not_transformed].dropna(),
             vary=vl,
@@ -377,7 +444,7 @@ def pred_plot_sk(
             maxq=maxq,
         )
         iplot_dum = dummify(iplot, transformed)[fn]
-        iplot["prediction"] = fitted.predict_proba(iplot_dum)[:, 1]
+        iplot["prediction"] = pred_fun(fitted, iplot_dum)
         if sum(is_num) < 2:
             min_max = calc_ylim("prediction", iplot, min_max)
         pred_dict[v] = iplot
@@ -385,26 +452,28 @@ def pred_plot_sk(
     row = col = 0
     for i, v in enumerate(incl):
         col = ifelse(i % 2 == 0, 0, 1)
-        if pd.api.types.is_numeric_dtype(df[v].dtype):
+        if pd.api.types.is_numeric_dtype(df[v].dtype) and df[v].nunique() > 5:
             fig = sns.lineplot(x=v, y="prediction", data=pred_dict[v], ax=ax[row, col])
         else:
             fig = sns.lineplot(
                 x=v, y="prediction", marker="o", data=pred_dict[v], ax=ax[row, col]
             )
-        # fig = sns.lineplot(x=v, y="prediction", data=pred_dict[v], ax=ax[row, col])
         if isinstance(min_max, tuple) and len(min_max) == 2:
-            ax[row, col].set(ylim=tuple(min_max))
+            ax[row, col].set(ylim=min_max)
         if hline != False:
             ax[row, col].axhline(y=hline, linestyle="--")
 
         if col == 1:
             row += 1
 
-    start_col = ifelse(col == 1, 0, 1)
+    start_col = ifelse((col == 1) or len(incl) == 0, 0, 1)
     for j, v in enumerate(incl_int):
         col = ifelse(j % 2 == start_col, 0, 1)
         vl = v.split(":")
-        is_num = [pd.api.types.is_numeric_dtype(df[c].dtype) for c in vl]
+        is_num = [
+            pd.api.types.is_numeric_dtype(df[c].dtype) and df[c].nunique() > 5
+            for c in vl
+        ]
         if sum(is_num) == 2:
             plot_data = (
                 pred_dict[v]
@@ -433,7 +502,7 @@ def pred_plot_sk(
             )
 
         if isinstance(min_max, tuple) and len(min_max) == 2 and sum(is_num) < 2:
-            ax[row, col].set(ylim=tuple(min_max))
+            ax[row, col].set(ylim=min_max)
         if sum(is_num) < 2 and hline != False:
             ax[row, col].axhline(y=hline, linestyle="--")
 
@@ -473,19 +542,23 @@ def vimp_plot_sm(fitted, df, rep=5):
     evars = extract_evars(model, df.columns)
     df = df[[rvar] + evars].copy().reset_index(drop=True).dropna()
 
+    if len(model.endog) != df.shape[0]:
+        raise Exception(
+            "The number of rows in the DataFrame should be the same as the number of rows in the data used to estimate the model"
+        )
+
     def imp_calc_reg(base, pred):
         return pd.DataFrame({"y": model.endog, "yhat": pred}).corr().iloc[0, 1] ** 2
 
     def imp_calc_logit(base, pred):
         return base - auc(model.endog, pred)
 
-    # Calculate the baseline mean squared error
+    # Calculate the baseline performance
     if isinstance(fitted, sm.genmod.generalized_linear_model.GLMResultsWrapper):
         baseline_fit = auc(model.endog, fitted.predict(df[evars]))
         imp_calc = imp_calc_logit  # specifying the function to use
         xlab = "Importance (AUC decrease)"
     elif isinstance(fitted, sm.regression.linear_model.RegressionResultsWrapper):
-        # baseline_fit = mean_squared_error(model.endog, fitted.predict(df[evars]))
         baseline_fit = (
             pd.DataFrame({"y": model.endog, "yhat": fitted.predict(df[evars])})
             .corr()
@@ -495,7 +568,7 @@ def vimp_plot_sm(fitted, df, rep=5):
         imp_calc = imp_calc_reg  # specifying the function to use
         xlab = "Importance (R-square decrease)"
     else:
-        return "Model for this model type not supported"
+        return "This model type is not supported. For sklearn models use vimp_plot_sk"
 
     # Create a copy of the dataframe
     permuted = df.copy()
@@ -517,7 +590,9 @@ def vimp_plot_sm(fitted, df, rep=5):
     importance_values = {k: [v / rep] for k, v in importance_values.items()}
     sorted_idx = pd.DataFrame(importance_values).transpose()
     sorted_idx = sorted_idx.sort_values(0, ascending=True)
-    fig = sorted_idx.plot.barh(color="slateblue", legend=None)
+    fig = sorted_idx.plot.barh(
+        color="slateblue", legend=None, figsize=(6, max(5, len(sorted_idx) * 0.5))
+    )
     plt.xlabel(xlab)
     plt.title("Permutation Importance")
 
@@ -530,9 +605,10 @@ def vimp_plot_sk(fitted, X, y, rep=5):
     Parameters
     ----------
     fitted : A fitted sklearn objects
-    X : Pandas DataFrame with data used for estimation
+    X : Pandas DataFrame with data containing the explanatory variables (features) used for estimation
+    y : Series with data for the response variable (target) used for estimation
     rep: int
-        The number of times to resample and calculate the permutation importance
+        The number of times to resample and calculate permutation importance
     """
 
     if hasattr(fitted, "classes_"):
@@ -548,6 +624,7 @@ def vimp_plot_sk(fitted, X, y, rep=5):
     data = pd.DataFrame(imp.importances.T)
     data.columns = fitted.feature_names_in_
     order = data.agg("mean").sort_values(ascending=False).index
+    plt.figure(figsize=(6, max(5, data.shape[0] * 1.5)))
     fig = sns.barplot(
         x="value",
         y="variable",
